@@ -34,35 +34,44 @@
 #include <set>
 #include <thread>
 
+#include <kodi/gui/dialogs/OK.h>
 #include <kodi/gui/dialogs/Progress.h>
 
 std::mutex WaipuData::mutex;
 
 // BEGIN CURL helpers from zattoo addon:
 std::string WaipuData::HttpGet(const std::string& url,
-                               const std::map<std::string, std::string>& headers)
+                               const std::map<std::string,
+                               std::string>& headers,
+			       bool ignoreStatusCode)
 {
-  return HttpRequest("GET", url, "", headers);
+  return HttpRequest("GET", url, "", headers, ignoreStatusCode);
 }
 
 std::string WaipuData::HttpDelete(const std::string& url,
                                   const std::string& postData,
-                                  const std::map<std::string, std::string>& headers)
+                                  const std::map<std::string,
+                                  std::string>& headers,
+				  bool ignoreStatusCode)
 {
-  return HttpRequest("DELETE", url, postData, headers);
+  return HttpRequest("DELETE", url, postData, headers, ignoreStatusCode);
 }
 
 std::string WaipuData::HttpPost(const std::string& url,
                                 const std::string& postData,
-                                const std::map<std::string, std::string>& headers)
+                                const std::map<std::string,
+                                std::string>& headers,
+				bool ignoreStatusCode)
 {
-  return HttpRequest("POST", url, postData, headers);
+  return HttpRequest("POST", url, postData, headers, ignoreStatusCode);
 }
 
 std::string WaipuData::HttpRequest(const std::string& action,
                                    const std::string& url,
                                    const std::string& postData,
-                                   const std::map<std::string, std::string>& headers)
+                                   const std::map<std::string,
+                                   std::string>& headers,
+                                   bool ignoreStatusCode)
 {
   Curl curl;
   int statusCode;
@@ -77,14 +86,15 @@ std::string WaipuData::HttpRequest(const std::string& action,
   curl.AddHeader("User-Agent", WAIPU_USER_AGENT);
   kodi::Log(ADDON_LOG_DEBUG, "HTTP User-Agent: %s.", WAIPU_USER_AGENT.c_str());
 
-  return HttpRequestToCurl(curl, action, url, postData, statusCode);
+  return HttpRequestToCurl(curl, action, url, postData, statusCode, ignoreStatusCode);
 }
 
 std::string WaipuData::HttpRequestToCurl(Curl& curl,
                                          const std::string& action,
                                          const std::string& url,
                                          const std::string& postData,
-                                         int& statusCode)
+                                         int& statusCode,
+                                         bool ignoreStatusCode)
 {
   kodi::Log(ADDON_LOG_DEBUG, "Http-Request: %s %s.", action.c_str(), url.c_str());
   std::string content;
@@ -104,7 +114,7 @@ std::string WaipuData::HttpRequestToCurl(Curl& curl,
   {
     content = curl.Get(url, statusCode);
   }
-  if (statusCode >= 200 && statusCode < 300)
+  if (ignoreStatusCode || (statusCode >= 200 && statusCode < 300))
     return content;
 
   kodi::Log(ADDON_LOG_ERROR, "[Http-GET-Request] error. status: %i, body: %s", statusCode,
@@ -576,6 +586,7 @@ void WaipuData::ReadSettings()
   m_channel_filter = kodi::addon::GetSettingEnum<WAIPU_CHANNEL_IMPORT_FILTER>(
       "channel_import_filter", CHANNEL_FILTER_ALL_VISIBLE);
   m_epg_show_preview_images = kodi::addon::GetSettingBoolean("epg_show_preview_images");
+  m_recordings_backend_handle_position = kodi::addon::GetSettingBoolean("recordings_backend_handle_position");
   m_refreshToken = JWT(kodi::addon::GetSettingString("refresh_token", ""));
 
   m_device_id = kodi::addon::GetSettingString("device_id_uuid4");
@@ -664,6 +675,14 @@ ADDON_STATUS WaipuData::SetSetting(const std::string& settingName,
   {
     kodi::addon::CInstancePVRClient::TriggerRecordingUpdate();
   }
+  else if (settingName == "recordings_backend_handle_position")
+  {
+    if (settingValue.GetBoolean() != m_recordings_backend_handle_position)
+      {
+	m_recordings_backend_handle_position = settingValue.GetBoolean();
+        return ADDON_STATUS_NEED_RESTART;
+      }
+  }
 
   return ADDON_STATUS_OK;
 }
@@ -676,7 +695,9 @@ PVR_ERROR WaipuData::GetCapabilities(kodi::addon::PVRCapabilities& capabilities)
   capabilities.SetSupportsRecordingsDelete(true);
   capabilities.SetSupportsTimers(true);
   capabilities.SetSupportsChannelGroups(true);
-  capabilities.SetSupportsLastPlayedPosition(true);
+
+  const bool handle_position = kodi::addon::GetSettingBoolean("recordings_backend_handle_position", false);
+  capabilities.SetSupportsLastPlayedPosition(handle_position);
 
   return PVR_ERROR_NO_ERROR;
 }
@@ -709,7 +730,7 @@ void WaipuData::SetStreamProperties(std::vector<kodi::addon::PVRStreamProperty>&
   properties.emplace_back(PVR_STREAM_PROPERTY_STREAMURL, url);
   properties.emplace_back(PVR_STREAM_PROPERTY_ISREALTIMESTREAM, realtime ? "true" : "false");
 
-  if (protocol == "dash" && Utils::CheckInputstreamInstalledAndEnabled("inputstream.adaptive"))
+  if ((protocol == "dash" || protocol == "MPEG_DASH") && Utils::CheckInputstreamInstalledAndEnabled("inputstream.adaptive"))
   {
     // MPEG DASH
     kodi::Log(ADDON_LOG_DEBUG, "[PLAY STREAM] dash");
@@ -730,7 +751,7 @@ void WaipuData::SetStreamProperties(std::vector<kodi::addon::PVRStreamProperty>&
                             "|Content-Type=text%2Fxml&x-dt-custom-data=" +
                                 license + "|R{SSM}|JBlicense");
   }
-  else if (protocol == "hls" && kodi::addon::GetSettingBoolean("streaming_use_ffmpegdirect", false))
+  else if ((protocol == "hls" || protocol == "HLS") && kodi::addon::GetSettingBoolean("streaming_use_ffmpegdirect", false))
   {
     if (!Utils::CheckInputstreamInstalledAndEnabled("inputstream.ffmpegdirect"))
     {
@@ -745,7 +766,7 @@ void WaipuData::SetStreamProperties(std::vector<kodi::addon::PVRStreamProperty>&
     properties.emplace_back("inputstream.ffmpegdirect.is_realtime_stream",
                             realtime ? "true" : "false");
   }
-  else if (protocol == "hls" && Utils::CheckInputstreamInstalledAndEnabled("inputstream.adaptive"))
+  else if ((protocol == "hls" || protocol == "HLS") && Utils::CheckInputstreamInstalledAndEnabled("inputstream.adaptive"))
   {
     kodi::Log(ADDON_LOG_DEBUG,
               "[SetStreamProperties] play protocol '%s' using inputstream adaptive",
@@ -774,6 +795,10 @@ bool WaipuData::LoadChannelData()
   if (m_channels.size() > 0)
     return true;
 
+  // user wishes no channels, maybe just uses the plugin for recordings?
+  if (m_channel_filter == CHANNEL_FILTER_NONE)
+    return true;
+
   // no valid session
   if (!IsConnected())
     return false;
@@ -1012,7 +1037,8 @@ std::string WaipuData::GetChannelStreamURL(int uniqueId,
       std::string jsonStreamURL = HttpPost(
           "https://stream-url-provider.waipu.tv/api/stream-url", postData,
           {{"Content-Type", "application/vnd.streamurlprovider.stream-url-request-v1+json"},
-           {"X-Device-Token", m_deviceCapabilitiesToken.getToken().c_str()}});
+           {"X-Device-Token", m_deviceCapabilitiesToken.getToken().c_str()}},
+          true);
 
       rapidjson::Document streamURLDoc;
       streamURLDoc.Parse(jsonStreamURL.c_str());
@@ -1022,9 +1048,20 @@ std::string WaipuData::GetChannelStreamURL(int uniqueId,
         return "";
       }
 
+      if (streamURLDoc.HasMember("status") &&
+          streamURLDoc["status"].GetInt() == 403 &&
+          streamURLDoc.HasMember("title") &&
+          streamURLDoc.HasMember("detail"))
+      {
+        const std::string errorTitle = streamURLDoc["title"].GetString();
+        const std::string errorDetail = streamURLDoc["detail"].GetString();
+        kodi::gui::dialogs::OK::ShowAndGetInput(errorTitle, errorDetail);
+        return "";
+      }
       if (!streamURLDoc.HasMember("streamUrl"))
       {
         kodi::Log(ADDON_LOG_ERROR, "[GetStreamURL] ERROR: missing param streamUrl");
+        kodi::Log(ADDON_LOG_ERROR, "[GetStreamURL] body: %s", jsonStreamURL.c_str());
         return "";
       }
 
@@ -1314,7 +1351,7 @@ PVR_ERROR WaipuData::GetEPGTagStreamProperties(
   if (protocol == "auto")
     protocol = "dash"; //fallback to dash
 
-  std::string strUrl = GetEPGTagURL(tag, protocol);
+  std::string strUrl = GetChannelStreamURL(tag.GetUniqueChannelId(), protocol, std::to_string(tag.GetStartTime()));
   if (strUrl.empty())
   {
     return PVR_ERROR_FAILED;
@@ -1325,72 +1362,6 @@ PVR_ERROR WaipuData::GetEPGTagStreamProperties(
   return PVR_ERROR_NO_ERROR;
 }
 
-std::string WaipuData::GetEPGTagURL(const kodi::addon::PVREPGTag& tag, const std::string& protocol)
-{
-  for (const auto& channel : m_channels)
-  {
-    if (channel.iUniqueId == tag.GetUniqueChannelId())
-    {
-      std::string startTime = Utils::TimeToString(tag.GetStartTime());
-      std::string endTime = Utils::TimeToString(tag.GetEndTime());
-
-      std::string jsonEpg =
-          HttpGet("https://epg.waipu.tv/api/channels/" + channel.waipuID +
-                  "/programs?includeRunningAtStartTime=false&startTime=" + std::string(startTime) +
-                  "&stopTime=" + std::string(endTime));
-      kodi::Log(ADDON_LOG_DEBUG, "[epg-single-tag] %s", jsonEpg.c_str());
-      if (jsonEpg.empty())
-      {
-        kodi::Log(ADDON_LOG_ERROR, "[epg-single-tag] empty server response");
-        return "";
-      }
-      jsonEpg = "{\"result\": " + jsonEpg + "}";
-
-      rapidjson::Document epgDoc;
-      epgDoc.Parse(jsonEpg.c_str());
-
-      if (epgDoc.HasParseError() || epgDoc["result"].Empty() ||
-          !epgDoc["result"][0].HasMember("streamUrlProvider") ||
-          epgDoc["result"][0]["streamUrlProvider"].IsNull())
-      {
-        // fallback to replay playback
-        kodi::Log(ADDON_LOG_DEBUG,
-                  "[play epg tag] streamUrlProvider not found -> fallback to replay!");
-        return GetChannelStreamURL(tag.GetUniqueChannelId(), protocol,
-                                   std::to_string(tag.GetStartTime()));
-      }
-
-      std::string url = epgDoc["result"][0]["streamUrlProvider"].GetString();
-
-      if (!url.empty())
-      {
-        kodi::Log(ADDON_LOG_DEBUG, "play url -> %s", url.c_str());
-
-        std::string tag_resp = HttpGet(url);
-        kodi::Log(ADDON_LOG_DEBUG, "tag resp -> %s", tag_resp.c_str());
-
-        rapidjson::Document tagDoc;
-        tagDoc.Parse(tag_resp.c_str());
-        if (tagDoc.HasParseError())
-        {
-          kodi::Log(ADDON_LOG_ERROR, "[getEPGTagURL] ERROR: error while parsing json");
-          return "";
-        }
-        kodi::Log(ADDON_LOG_DEBUG, "[tag] streams");
-        // check if streams there
-        if (tagDoc.HasMember("player") && tagDoc["player"].HasMember("mpd"))
-        {
-          std::string mpdUrl = tagDoc["player"]["mpd"].GetString();
-          kodi::Log(ADDON_LOG_DEBUG, "mpd url -> %s", mpdUrl.c_str());
-          return mpdUrl;
-        }
-      }
-    }
-  }
-  kodi::Log(ADDON_LOG_DEBUG, "[play epg tag] channel or tag not found!");
-  return "";
-}
-
 PVR_ERROR WaipuData::GetRecordingsAmount(bool deleted, int& amount)
 {
   if (!IsConnected())
@@ -1636,7 +1607,7 @@ std::string WaipuData::GetRecordingURL(const kodi::addon::PVRRecording& recordin
   std::string recording_id = recording.GetRecordingId();
   kodi::Log(ADDON_LOG_DEBUG, "play recording -> %s", recording_id.c_str());
 
-  std::string rec_resp = HttpGet("https://recording.waipu.tv/api/recordings/" + recording_id);
+  std::string rec_resp = HttpGet("https://recording.waipu.tv/api/recordings/" + recording_id + "/streamingdetails");
   kodi::Log(ADDON_LOG_DEBUG, "recording resp -> %s", rec_resp.c_str());
 
   rapidjson::Document recordingDoc;
@@ -1648,18 +1619,18 @@ std::string WaipuData::GetRecordingURL(const kodi::addon::PVRRecording& recordin
   }
   kodi::Log(ADDON_LOG_DEBUG, "[recording] streams");
   // check if streams there
-  if (!recordingDoc.HasMember("streamingDetails") ||
-      !recordingDoc["streamingDetails"].HasMember("streams"))
+  if (!recordingDoc.HasMember("streams"))
   {
+    kodi::Log(ADDON_LOG_ERROR, "[getRecordingURL] ERROR: missing streams");
     return "";
   }
 
   kodi::Log(ADDON_LOG_DEBUG, "[recordings] size: %i;",
-            recordingDoc["streamingDetails"]["streams"].Size());
+            recordingDoc["streams"].Size());
 
   std::string protocol_fix = protocol == "dash" ? "MPEG_DASH" : "HLS";
 
-  for (const auto& stream : recordingDoc["streamingDetails"]["streams"].GetArray())
+  for (const auto& stream : recordingDoc["streams"].GetArray())
   {
     std::string current_protocol = stream["protocol"].GetString();
     kodi::Log(ADDON_LOG_DEBUG, "[stream] protocol: %s;", current_protocol.c_str());
@@ -1746,7 +1717,7 @@ PVR_ERROR WaipuData::GetTimers(kodi::addon::PVRTimersResultSet& results)
   LoadChannelData();
 
   std::string jsonRecordings = HttpGet("https://recording.waipu.tv/api/recordings",
-                                       {{"Accept", "application/vnd.waipu.recordings-v2+json"}});
+                                       {{"Accept", "application/vnd.waipu.recordings-v4+json"}});
   kodi::Log(ADDON_LOG_DEBUG, "[Timers] %s", jsonRecordings.c_str());
 
   jsonRecordings = "{\"result\": " + jsonRecordings + "}";
@@ -1769,9 +1740,9 @@ PVR_ERROR WaipuData::GetTimers(kodi::addon::PVRTimersResultSet& results)
   for (const auto& timer : timersDoc["result"].GetArray())
   {
     // skip if missing epgdata
-    if (!timer.HasMember("epgData"))
+    if (!timer.HasMember("status") || !timer.HasMember("stationId") || !timer.HasMember("title"))
     {
-      kodi::Log(ADDON_LOG_DEBUG, "[timers] Skip due to missing epgData");
+      kodi::Log(ADDON_LOG_DEBUG, "[timers] Skip due to missing status/station/title");
       continue;
     }
 
@@ -1797,21 +1768,25 @@ PVR_ERROR WaipuData::GetTimers(kodi::addon::PVRTimersResultSet& results)
     }
     tag.SetLifetime(0);
 
-    const rapidjson::Value& epgData = timer["epgData"];
 
     // set recording title
-    std::string rec_title = epgData["title"].GetString();
+    std::string rec_title = timer["title"].GetString();
     kodi::Log(ADDON_LOG_DEBUG, "[timers] Add: %s;", rec_title.c_str());
     tag.SetTitle(rec_title);
 
     int tag_channel;
     // channelid
-    if (timer.HasMember("channelId") && !timer["channelId"].IsNull())
+    if (timer.HasMember("stationId") && !timer["stationId"].IsNull())
     {
-      std::string channel_name = timer["channelId"].GetString();
+      std::string station_id = timer["stationId"].GetString();
+
+      // workaround: transform Station ID to uppercase, since old API (for recordings/timers) needs this
+      std::transform(station_id.begin(), station_id.end(),
+		     station_id.begin(), ::toupper);
+
       for (const auto& channel : m_channels)
       {
-        if (channel.waipuID != channel_name)
+        if (channel.waipuID != station_id)
           continue;
         tag_channel = channel.iUniqueId;
         tag.SetClientChannelUid(tag_channel);
@@ -1848,28 +1823,28 @@ PVR_ERROR WaipuData::GetTimers(kodi::addon::PVRTimersResultSet& results)
     tag.SetEPGUid(Utils::StringToInt(rec_id, 0));
 
     // get recording time
-    if (timer.HasMember("startTime") && !timer["startTime"].IsNull())
+    if (timer.HasMember("epgStartTime") && !timer["epgStartTime"].IsNull())
     {
-      std::string startTime = timer["startTime"].GetString();
+      std::string startTime = timer["epgStartTime"].GetString();
       tag.SetStartTime(Utils::StringToTime(startTime));
     }
-    if (timer.HasMember("stopTime") && !timer["stopTime"].IsNull())
-    {
-      std::string endTime = timer["stopTime"].GetString();
-      tag.SetEndTime(Utils::StringToTime(endTime));
-    }
+    //if (timer.HasMember("stopTime") && !timer["stopTime"].IsNull())
+    //{
+    //  std::string endTime = timer["stopTime"].GetString();
+    //  tag.SetEndTime(Utils::StringToTime(endTime));
+    //}
 
     // get plot
-    if (epgData.HasMember("description") && !epgData["description"].IsNull())
-    {
-      std::string rec_plot = epgData["description"].GetString();
-      tag.SetSummary(rec_plot);
-    }
+    //if (epgData.HasMember("description") && !epgData["description"].IsNull())
+    //{
+    //  std::string rec_plot = epgData["description"].GetString();
+    //  tag.SetSummary(rec_plot);
+    //}
 
     // epg mapping
-    if (epgData.HasMember("id") && !epgData["id"].IsNull())
+    if (timer.HasMember("programId") && !timer["programId"].IsNull())
     {
-      std::string epg_id = epgData["id"].GetString();
+      std::string epg_id = timer["programId"].GetString();
       int dirtyID = Utils::GetIDDirty(epg_id);
       tag.SetEPGUid(dirtyID);
     }
